<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

/*
 * Load Additional Data when getting Order by OrderRepository 
 */
class OrderGet
{
    /** @var \FiloBlu\OrderAdditionalData\Model\OrderDataFactory */
    protected $orderDataFactory;

    /** @var \FiloBlu\OrderAdditionalData\Model\ItemDataFactory */
    protected $itemDataFactory;

    /** @var \Magento\Sales\Api\Data\OrderExtensionFactory */
    protected $orderExtensionFactory;

    /** @var \Magento\Sales\Api\Data\OrderItemExtensionFactory */
    protected $orderItemExtensionFactory;
    
    /**
     * Init plugin
     *
     * @param \FiloBlu\OrderAdditionalData\Model\OrderDataFactory $orderDataFactory
     * @param \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory
     * @param \Magento\Sales\Api\Data\OrderItemExtensionFactory $orderItemExtensionFactory     
     */
    public function __construct(
        \FiloBlu\OrderAdditionalData\Model\OrderDataFactory $orderDataFactory,
        \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory,
        \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
        \Magento\Sales\Api\Data\OrderItemExtensionFactory $orderItemExtensionFactory
    ) {
        $this->orderDataFactory = $orderDataFactory;
        $this->itemDataFactory = $itemDataFactory;
        $this->orderExtensionFactory = $orderExtensionFactory;
        $this->orderItemExtensionFactory = $orderItemExtensionFactory;
    }

    /**
     * Get gift message
     *
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderInterface $resultOrder
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    public function afterGet(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderInterface $resultOrder
    ) {
        $resultOrder = $this->getOrderAdditionalData($resultOrder);
        $resultOrder = $this->getOrderItemAdditionalData($resultOrder);

        return $resultOrder;
    }

    /**
     * Get gift message for order
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    protected function getOrderAdditionalData(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $extensionAttributes = $order->getExtensionAttributes();
        if ($extensionAttributes && $extensionAttributes->getFbAdditionalData()) {
            return $order;
        }
        //$additionalData=$this->objectFactory->create();
        $additionalData=array();
        try {
            /** @var \Magento\AdditionalData\Api\Data\MessageInterface $AdditionalData */
            $orderDataObject = $this->orderDataFactory->create()->load($order->getEntityId(), "order_id");

            if ($orderDataObject && $orderDataObject->getID() > 0 && $orderDataObject->getAdditionalData()!=null ){
                $additionalData=unserialize($orderDataObject->getAdditionalData());
            }
        } catch (NoSuchEntityException $e) {
            return $order;
        }

        /** @var \Magento\Sales\Api\Data\OrderExtension $orderExtension */
        $orderExtension = $extensionAttributes ? $extensionAttributes : $this->orderExtensionFactory->create();
        $orderExtension->setFbAdditionalData(array($additionalData));
        $order->setExtensionAttributes($orderExtension);

        return $order;
    }

    /**
     * Get additional data for items of order
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    protected function getOrderItemAdditionalData(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $orderItems = $order->getItems();
        if (null !== $orderItems) {
            /** @var \Magento\Sales\Api\Data\OrderItemInterface $orderItem */
            foreach ($orderItems as $orderItem) {
                $extensionAttributes = $orderItem->getExtensionAttributes();
                if ($extensionAttributes && $extensionAttributes->getFbAdditionalData()) {
                    continue;
                }

                try {
                    /* @var \Magento\GiftMessage\Api\Data\MessageInterface $giftMessage */
                    $itemDataObject = $this->itemDataFactory->create()->load($orderItem->getItemId(), "item_id");
                    $additionalData=unserialize($itemDataObject->getAdditionalData());

                } catch (NoSuchEntityException $e) {
                    continue;
                }

                /** @var \Magento\Sales\Api\Data\OrderItemExtension $orderItemExtension */
                $orderItemExtension = $extensionAttributes
                    ? $extensionAttributes
                    : $this->orderItemExtensionFactory->create();
                $orderItemExtension->setFbAdditionalData(array($additionalData));
                $orderItem->setExtensionAttributes($orderItemExtension);
            }
        }
        return $order;
    }

    /**
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Model\ResourceModel\Order\Collection $resultOrder
     * @return \Magento\Sales\Model\ResourceModel\Order\Collection
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetList(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Model\ResourceModel\Order\Collection $resultOrder
    ) {
        /** @var  $order */
        foreach ($resultOrder->getItems() as $order) {
            $this->afterGet($subject, $order);
        }
        return $resultOrder;
    }
}
