<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

use Magento\Framework\Exception\CouldNotSaveException;
/*
 * Save Additional Data when saving Orders by OrderRepository 
 */
class OrderSave
{
    /** @var \FiloBlu\OrderAdditionalData\Model\OrderDataFactory */
    protected $orderDataFactory;

    /** @var \Magento\GiftMessage\Api\OrderItemRepositoryInterface */
    protected $orderItemRepository;

    /**
     * Init plugin
     *
     * @param \Magento\GiftMessage\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\GiftMessage\Api\OrderItemRepositoryInterface $orderItemRepository
     */
    public function __construct(
        \FiloBlu\OrderAdditionalData\Model\OrderDataFactory $orderDataFactory
    ) {
        $this->orderDataFactory = $orderDataFactory;
    }

    /**
     * Save gift message
     *
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderInterface $resultOrder
     * @return \Magento\Sales\Api\Data\OrderInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @throws CouldNotSaveException
     */
    public function afterSave(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderInterface $resultOrder
    ) {
        /** @var \Magento\Sales\Api\Data\OrderInterface $resultOrder */
        $resultOrder = $this->saveOrderAdditionalData($resultOrder);

        return $resultOrder;
    }

    /**
     * Save additional data for order
     *
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return \Magento\Sales\Api\Data\OrderInterface
     * @throws CouldNotSaveException
     */
    protected function saveOrderAdditionalData(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $extensionAttributes = $order->getExtensionAttributes();
        if (
            null !== $extensionAttributes &&
            null !== $extensionAttributes->getFbAdditionalData()
        ) {
            
            $additionalData = current($extensionAttributes->getFbAdditionalData());
            try {
                $orderData=$this->orderDataFactory->create()->load($order->getEntityId(), "order_id");
                $orderData->setOrderId($order->getEntityId());
                $orderData->setAdditionalData(serialize($additionalData));
                $orderData->save();                
            } catch (\Exception $e) {
                throw new CouldNotSaveException(
                    __('Could not save Additional Data for order: "%1"', $e->getMessage()),
                    $e
                );
            }
        }
        return $order;
    }

}
